@@ -1,21 +1,23 @@
-/* Copyright (C) 2015 J.F.Dockes
+/* Copyright (C) 2015-2020 J.F.Dockes
  *   This program is free software; you can redistribute it and/or modify
- *   it under the terms of the GNU General Public License as published by
- *   the Free Software Foundation; either version 2 of the License, or
+ *   it under the terms of the GNU Lesser General Public License as published by
+ *   the Free Software Foundation; either version 2.1 of the License, or
  *   (at your option) any later version.
  *
  *   This program is distributed in the hope that it will be useful,
  *   but WITHOUT ANY WARRANTY; without even the implied warranty of
  *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *   GNU General Public License for more details.
+ *   GNU Lesser General Public License for more details.
  *
- *   You should have received a copy of the GNU General Public License
+ *   You should have received a copy of the GNU Lesser General Public License
  *   along with this program; if not, write to the
  *   Free Software Foundation, Inc.,
  *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 /////////////////////////////////////////////////////////////////////
 // Main program
+#define UPMPDCLI_NEED_PACKAGE_VERSION
+#include "config.h"
 
 #include <errno.h>             
 #include <fcntl.h>             
@@ -26,6 +28,7 @@
 #include <sys/param.h>         
 #include <unistd.h>            
 #include <grp.h>
+#include <assert.h>
 
 #include <iostream>            
 #include <string>              
@@ -35,31 +38,38 @@
 #include "libupnpp/log.hxx"    
 #include "libupnpp/upnpplib.hxx"
 #include "execmd.h"
-#include "conftree.hxx"
+#include "conftree.h"
 #include "mpdcli.hxx"
 #include "upmpd.hxx"
-#include "httpfs.hxx"
+#include "mediaserver/mediaserver.hxx"
+#include "mediaserver/contentdirectory.hxx"
 #include "upmpdutils.hxx"
+#include "pathut.h"
+#include "readfile.h"
 
 using namespace std;
 using namespace UPnPP;
 
+// Can't remember why it's static in config.h and don't want to try change
+const string g_upmpdcli_package_version{UPMPDCLI_PACKAGE_VERSION};
+
 static char *thisprog;
 
 static int op_flags;
-#define OPT_MOINS 0x1
-#define OPT_h     0x2
-#define OPT_p     0x4
-#define OPT_d     0x8
-#define OPT_D     0x10
-#define OPT_c     0x20
-#define OPT_l     0x40
-#define OPT_f     0x80
-#define OPT_q     0x100
-#define OPT_i     0x200
-#define OPT_P     0x400
-#define OPT_O     0x800
-#define OPT_v     0x1000
+#define OPT_MOINS 0x1   
+#define OPT_D     0x2   
+#define OPT_O     0x4   
+#define OPT_P     0x8   
+#define OPT_c     0x10  
+#define OPT_d     0x20  
+#define OPT_f     0x40  
+#define OPT_h     0x80  
+#define OPT_i     0x100 
+#define OPT_l     0x200 
+#define OPT_m     0x400 
+#define OPT_p     0x800 
+#define OPT_q     0x1000
+#define OPT_v     0x2000
 
 static const char usage[] = 
     "-c configfile \t configuration file to use\n"
@@ -74,9 +84,29 @@ static const char usage[] =
     "-P upport    \t specify port number to be used for UPnP\n"
     "-O 0|1\t decide if we run and export the OpenHome services\n"
     "-v      \tprint version info\n"
+    "-m <0|1|2|3|4> media server mode "
+    "(default, multidev|only renderer|only media|embedded|multidev)\n"
     "\n"
     ;
 
+// We can implement a Media Server in addition to the Renderer, for
+// accessing streaming services. This can happen in several modes. In
+// all cases, the Media Server is only created if the configuration
+// file does have parameters set for streaming services.
+// - -m 0, -m 4 combined/multidev: implement the Media Server and
+//   Media Renderers, as enabled by the configuration, as separate
+//   root devices.
+// - -m 1, RdrOnly: for the main instance: be a Renderer, do not start the
+//    Media Server even if the configuration indicates it is needed
+//    (this is not used in normal situations, just edit the config
+//    instead!)
+// - -m 2, MSOnly Media Server only.
+// - -m 3, combined/embedded: implement the Media Server
+//    as an embedded device. This works just fine with, for example,
+//    upplay, but confuses most of the other Control Points.
+
+enum MSMode {Default, RdrOnly, MSOnly, CombinedEmbedded, CombinedMultiDev};
+
 static void
 versionInfo(FILE *fp)
 {
@@ -113,29 +143,35 @@ ohProductDesc_t ohProductDesc = {
     // Product
     {
         "Upmpdcli",                                 // name
-        PACKAGE_VERSION,                            // info
+        UPMPDCLI_PACKAGE_VERSION,                   // info
         "",                                         // url
         ""                                          // imageUri
     }
 };
 
-// This is global
-string g_protocolInfo;
-
 // Static for cleanup in sig handler.
-static UpnpDevice *dev;
+static vector<UpnpDevice *> devs;
+static MPDCli *mpdclip{nullptr};
 
 string g_datadir(DATADIR "/");
+string g_cachedir("/var/cache/upmpdcli");
 
 // Global
-string g_configfilename(CONFIGDIR "/upmpdcli.conf");
-PTMutexInit g_configlock;
+string g_configfilename;
 ConfSimple *g_config;
+ConfSimple *g_state;
+bool g_enableL16 = false;
+bool g_lumincompat = false;
 
 static void onsig(int)
 {
     LOGDEB("Got sig" << endl);
-    dev->shouldExit();
+    for (auto& dev : devs) {
+        dev->shouldExit();
+    }
+    if (mpdclip) {
+        mpdclip->shouldExit();
+    }
 }
 
 static const int catchedSigs[] = {SIGINT, SIGQUIT, SIGTERM};
@@ -153,10 +189,35 @@ static void setupsigs()
         }
 }
 
+static UpnpDevice *rootdevice{nullptr};
+static MediaServer *mediaserver{nullptr};
+static std::string uuidMS;
+static std::string fnameMS;
+static bool msroot{false};
+bool startMediaServer(bool enable)
+{
+    if (mediaserver) {
+        return true;
+    }
+    mediaserver = new MediaServer(
+        msroot ? nullptr : rootdevice, string("uuid:")+ uuidMS, fnameMS, enable);
+    if (nullptr == mediaserver)
+        return false;
+    devs.push_back(mediaserver);
+    LOGDEB("Media server event loop" << endl);
+    // msonly && !enableMediaServer is possible if we're just using
+    // the "mediaserver" to redirect URLs for ohcredentials/Kazoo
+    if (enable) {
+        mediaserver->startloop();
+    }
+    return true;
+}
+
 int main(int argc, char *argv[])
 {
     // Path for the sc2mpd command, or empty
     string sc2mpdpath;
+    string screceiverstatefile;
 
     // Sender mode: path for the command creating the mpd and mpd2sc
     // processes, and port for the auxiliary mpd.
@@ -174,6 +235,7 @@ int main(int argc, char *argv[])
     bool ownqueue = true;
     bool enableAV = true;
     bool enableOH = true;
+    bool enableMediaServer = false;
     bool ohmetapersist = true;
     string upmpdcliuser("upmpdcli");
     string pidfilename("/var/run/upmpdcli.pid");
@@ -182,7 +244,10 @@ int main(int argc, char *argv[])
     string iface;
     unsigned short upport = 0;
     string upnpip;
-
+    int msm = 0;
+    bool inprocessms{false};
+    bool msonly{false};
+    
     const char *cp;
     if ((cp = getenv("UPMPD_HOST")))
         mpdhost = cp;
@@ -218,6 +283,8 @@ int main(int argc, char *argv[])
                 iface = *(++argv); argc--; goto b1;
             case 'l':   op_flags |= OPT_l; if (argc < 2)  Usage();
                 loglevel = atoi(*(++argv)); argc--; goto b1;
+            case 'm':   op_flags |= OPT_m; if (argc < 2)  Usage();
+                msm = atoi(*(++argv)); argc--; goto b1;
             case 'O': {
                 op_flags |= OPT_O; 
                 if (argc < 2)  Usage();
@@ -239,12 +306,13 @@ int main(int argc, char *argv[])
     b1: argc--; argv++;
     }
 
-    if (argc != 0)
+    if (argc != 0 || msm < 0 || msm > 4) {
         Usage();
-
+    }
+    MSMode arg_msmode = MSMode(msm);
+    
     UpMpd::Options opts;
 
-    string cachedir;
     if (!g_configfilename.empty()) {
         g_config = new ConfSimple(g_configfilename.c_str(), 1, true);
         if (!g_config || !g_config->ok()) {
@@ -257,34 +325,38 @@ int main(int argc, char *argv[])
             g_config->get("logfilename", logfilename);
         if (!(op_flags & OPT_f))
             g_config->get("friendlyname", friendlyname);
-        if (!(op_flags & OPT_l) && g_config->get("loglevel", value))
-            loglevel = atoi(value.c_str());
-        if (!(op_flags & OPT_h))
+        if (!(op_flags & OPT_l)) {
+            loglevel = g_config->getInt("loglevel", Logger::LLINF);
+        }
+        if (!(op_flags & OPT_h)) {
             g_config->get("mpdhost", mpdhost);
-        if (!(op_flags & OPT_p) && g_config->get("mpdport", value)) {
-            mpdport = atoi(value.c_str());
+        }
+        if (!(op_flags & OPT_p)) {
+            mpdport =  g_config->getInt("mpdport", 6600);
         }
         g_config->get("mpdpassword", mpdpassword);
-        if (!(op_flags & OPT_q) && g_config->get("ownqueue", value)) {
-            ownqueue = atoi(value.c_str()) != 0;
+        if (!(op_flags & OPT_q)) {
+            ownqueue = g_config->getBool("ownqueue", true);
         }
-        if (g_config->get("openhome", value)) {
-            enableOH = atoi(value.c_str()) != 0;
-        }
-        if (g_config->get("upnpav", value)) {
-            enableAV = atoi(value.c_str()) != 0;
-        }
-        if (g_config->get("ohmetapersist", value)) {
-            ohmetapersist = atoi(value.c_str()) != 0;
+        enableOH = g_config->getBool("openhome", true);
+        enableAV = g_config->getBool("upnpav", true);
+
+        if (!g_config->getBool("checkcontentformat", true)) {
+            // If option is specified and 0, set nocheck flag
+            opts.options |= UpMpd::upmpdNoContentFormatCheck;
         }
+        ohmetapersist = g_config->getBool("ohmetapersist", true);
         if (g_config->get("pkgdatadir", g_datadir)) {
             path_catslash(g_datadir);
             iconpath = path_cat(g_datadir, "icon.png");
+            if (!path_exists(iconpath)) {
+                iconpath.clear();
+            }
             presentationhtml = path_cat(g_datadir, "presentation.html");
         }
         g_config->get("iconpath", iconpath);
         g_config->get("presentationhtml", presentationhtml);
-        g_config->get("cachedir", cachedir);
+        g_config->get("cachedir", opts.cachedir);
         g_config->get("pidfile", pidfilename);
         if (!(op_flags & OPT_i)) {
             g_config->get("upnpiface", iface);
@@ -292,15 +364,18 @@ int main(int argc, char *argv[])
                 g_config->get("upnpip", upnpip);
             }
         }
-        if (!(op_flags & OPT_P) && g_config->get("upnpport", value)) {
-            upport = atoi(value.c_str());
+        if (!(op_flags & OPT_P)) {
+            upport = g_config->getInt("upnpport", 0);
         }
-        if (g_config->get("schttpport", value))
-            opts.schttpport = atoi(value.c_str());
+        opts.schttpport = g_config->getInt("schttpport", 0);
         g_config->get("scplaymethod", opts.scplaymethod);
         g_config->get("sc2mpd", sc2mpdpath);
-        if (g_config->get("ohmetasleep", value))
-            opts.ohmetasleep = atoi(value.c_str());
+        g_config->get("screceiverstatefile", screceiverstatefile);
+        if (g_config->getBool("scnosongcastsource", false)) {
+            // If option is specified and 1, set nocheck flag
+            opts.options |= UpMpd::upmpdNoSongcastSource;
+        }
+        opts.ohmetasleep = g_config->getInt("ohmetasleep", 0);
         g_config->get("ohmanufacturername", ohProductDesc.manufacturer.name);
         g_config->get("ohmanufacturerinfo", ohProductDesc.manufacturer.info);
         g_config->get("ohmanufacturerurl", ohProductDesc.manufacturer.url);
@@ -326,21 +401,72 @@ int main(int argc, char *argv[])
         g_config->get("scsenderpath", senderpath);
         if (g_config->get("scsendermpdport", value))
             sendermpdport = atoi(value.c_str());
+
+        g_lumincompat = g_config->getBool("lumincompat", false);
+    } else {
+        // g_configfilename is empty. Create an empty config anyway
+        g_config = new ConfSimple(string(), 1, true);
+        if (!g_config || !g_config->ok()) {
+            cerr << "Could not create empty config\n";
+            return 1;
+        }
     }
+
     if (Logger::getTheLog(logfilename) == 0) {
         cerr << "Can't initialize log" << endl;
         return 1;
     }
+    Logger::getTheLog("")->reopen(logfilename);
     Logger::getTheLog("")->setLogLevel(Logger::LogLevel(loglevel));
 
+    // If a streaming service is enabled, we need a Media
+    // Server. We let a static ContentDirectory method decide this
+    // for us. The way we then implement it depends on the command
+    // line option (see the enum comments near the top of the file):
+    enableMediaServer = ContentDirectory::mediaServerNeeded();
+    switch (arg_msmode) {
+    case MSOnly:
+        inprocessms = true;
+        msonly = true;
+        break;
+    case CombinedEmbedded:
+        inprocessms = true;
+        msonly = false;
+        msroot = false;
+        break;
+    case RdrOnly:
+        inprocessms = false;
+        msonly = false;
+        break;
+    case CombinedMultiDev:
+    case Default:
+    default:
+        inprocessms = true;
+        msonly = false;
+        msroot = true;
+        break;
+    }
+
+    // If neither OH nor AV are enabled, run as pure media server. This
+    // is another way to do it besides the -m option
+    if (!enableOH && !enableAV) {
+        msonly = true;
+        inprocessms = true;
+    }
+    
     Pidfile pidfile(pidfilename);
 
-    // If started by root, do the pidfile + change uid thing
+    // If started by root, we use the pidfile and we will change the
+    // uid (later). First part follows
     uid_t runas(0);
     gid_t runasg(0);
+    struct passwd *pass = getpwnam(upmpdcliuser.c_str());
+    if (pass) {
+        runas = pass->pw_uid;
+        runasg = pass->pw_gid;
+    }
     if (geteuid() == 0) {
-        struct passwd *pass = getpwnam(upmpdcliuser.c_str());
-        if (pass == 0) {
+        if (runas == 0) {
             LOGFAT("upmpdcli won't run as root and user " << upmpdcliuser << 
                    " does not exist " << endl);
             return 1;
@@ -358,31 +484,45 @@ int main(int argc, char *argv[])
             LOGFAT("Can't write pidfile: " << pidfile.getreason() << endl);
             return 1;
         }
-	if (cachedir.empty())
-            cachedir = "/var/cache/upmpdcli";
+    if (opts.cachedir.empty())
+            opts.cachedir = "/var/cache/upmpdcli";
+    } else if (runas == geteuid()) {
+        // Already running as upmpdcli user. There are actually 2
+        // possibilities: either we were initially started as
+        // upmpdcli, and we should be using ~upmpdcli/.cache as work
+        // directory, or we were exec'd by a master process started as
+        // root, and we should be using /var/cache/upmpdcli. We have
+        // no way to decide actually, another command line option
+        // would be needed. For now, behave as if exec'd by a master
+        // process, because this is what happens if the default
+        // package was installed. One way to fix this if this if it is
+        // wrong is to set cachedir in the configuration file
+        // (opts.cachedir will be non-empty then).
+    if (opts.cachedir.empty())
+            opts.cachedir = "/var/cache/upmpdcli";
     } else {
-	if (cachedir.empty())
-            cachedir = path_cat(path_tildexpand("~") , "/.cache/upmpdcli");
+    if (opts.cachedir.empty())
+            opts.cachedir = path_cat(path_tildexpand("~") , "/.cache/upmpdcli");
     }
 
-    string& mcfn = opts.cachefn;
-    if (ohmetapersist) {
-        opts.cachefn = path_cat(cachedir, "/metacache");
-        if (!path_makepath(cachedir, 0755)) {
-            LOGERR("makepath("<< cachedir << ") : errno : " << errno << endl);
+    g_cachedir = opts.cachedir;
+    if (!path_makepath(opts.cachedir, 0755)) {
+        LOGERR("makepath("<< opts.cachedir << ") : errno : " << errno << endl);
+        cerr << "Can't create " << opts.cachedir << endl;
+        return 1;
+    }
+
+    string statefn = path_cat(opts.cachedir, "/upmstate");
+    g_state = new ConfSimple(statefn.c_str());
+    
+    opts.cachefn.clear();
+    if (!msonly && ohmetapersist) {
+        opts.cachefn = path_cat(opts.cachedir, "/metacache");
+        int fd;
+        if ((fd = open(opts.cachefn.c_str(), O_CREAT|O_RDWR, 0644)) < 0) {
+            LOGERR("creat("<< opts.cachefn << ") : errno : " << errno << endl);
         } else {
-            int fd;
-            if ((fd = open(mcfn.c_str(), O_CREAT|O_RDWR, 0644)) < 0) {
-                LOGERR("creat("<< mcfn << ") : errno : " << errno << endl);
-            } else {
-                close(fd);
-                if (geteuid() == 0 && chown(mcfn.c_str(), runas, -1) != 0) {
-                    LOGERR("chown("<< mcfn << ") : errno : " << errno << endl);
-                }
-                if (geteuid() == 0 && chown(cachedir.c_str(), runas, -1) != 0) {
-                    LOGERR("chown("<< cachedir << ") : errno : " << errno << endl);
-                }
-            }
+            close(fd);
         }
     }
     
@@ -394,13 +534,30 @@ int main(int argc, char *argv[])
     }
 
     if (geteuid() == 0) {
-        // Need to rewrite pid, it may have changed with the daemon call
+        // Need to rewrite pid, it may have changed with the daemon call. Also
+        // adjust file ownership and access.
         pidfile.write_pid();
         if (!logfilename.empty() && logfilename.compare("stderr")) {
-            if (chown(logfilename.c_str(), runas, -1) < 0) {
+            if (chown(logfilename.c_str(), runas, -1) < 0 && errno != ENOENT) {
                 LOGERR("chown("<<logfilename<<") : errno : " << errno << endl);
             }
         }
+        if (chown(opts.cachedir.c_str(), runas, -1) != 0) {
+            LOGERR("chown("<< opts.cachedir << ") : errno : " << errno << endl);
+        }
+        if (chown(statefn.c_str(), runas, -1) != 0) {
+            LOGERR("chown("<< statefn << ") : errno : " << errno << endl);
+        }
+        if (!opts.cachefn.empty()) {
+            if (chown(opts.cachefn.c_str(), runas, -1) != 0) {
+                LOGERR("chown("<< opts.cachefn << ") : errno : " <<
+                       errno << endl);
+            }
+        }
+        if (!g_configfilename.empty()) {
+            ensureconfreadable(g_configfilename.c_str(), upmpdcliuser.c_str(),
+                               runas, runasg);
+        }
         if (initgroups(upmpdcliuser.c_str(), runasg) < 0) {
             LOGERR("initgroup failed. Errno: " << errno << endl);
         }
@@ -409,7 +566,7 @@ int main(int argc, char *argv[])
                    << endl);
             return 1;
         }
-#if 0        
+#if 0
         gid_t list[100];
         int ng = getgroups(100, list);
         cerr << "GROUPS: ";
@@ -420,7 +577,8 @@ int main(int argc, char *argv[])
 #endif
     }
 
-//// Dropped root 
+
+/////////////////////////// Dropped root /////////////////////////////
 
     if (sc2mpdpath.empty()) {
         // Do we have an sc2mpd command installed (for songcast)?
@@ -464,46 +622,42 @@ int main(int argc, char *argv[])
 
 
     // Initialize MPD client object. Retry until it works or power fail.
-    MPDCli *mpdclip = 0;
-    int mpdretrysecs = 2;
-    for (;;) {
-        mpdclip = new MPDCli(mpdhost, mpdport, mpdpassword);
-        if (mpdclip == 0) {
-            LOGFAT("Can't allocate MPD client object" << endl);
-            return 1;
+    if (!msonly) {
+        int mpdretrysecs = 2;
+        for (;;) {
+            mpdclip = new MPDCli(mpdhost, mpdport, mpdpassword);
+            if (mpdclip == 0) {
+                LOGFAT("Can't allocate MPD client object" << endl);
+                return 1;
+            }
+            if (!mpdclip->ok()) {
+                LOGERR("MPD connection failed" << endl);
+                delete mpdclip;
+                mpdclip = 0;
+                sleep(mpdretrysecs);
+                mpdretrysecs = MIN(2*mpdretrysecs, 120);
+            } else {
+                break;
+            }
         }
-        if (!mpdclip->ok()) {
-            LOGERR("MPD connection failed" << endl);
-            delete mpdclip;
-            mpdclip = 0;
-            sleep(mpdretrysecs);
-            mpdretrysecs = MIN(2*mpdretrysecs, 120);
-        } else {
-            break;
+        const MpdStatus& mpdstat = mpdclip->getStatus();
+        // Only the "special" upmpdcli 0.19.16 version has patch != 0
+        g_enableL16 = mpdstat.versmajor >= 1 || mpdstat.versminor >= 20 ||
+            mpdstat.verspatch >= 16;
+        // Also L16 is a major source of issues when playing with
+        // win10 'cast to device', inciting it to transcode for some
+        // reason, with very bad results. So for the future (new in
+        // 1.5), only enable it if it's explicitely required by the
+        // config.
+        bool confl16{false};
+        if (g_config) {
+            confl16 = g_config->getBool("enablel16", false);
         }
+        g_enableL16 = g_enableL16 && confl16;
     }
-
-    // Initialize libupnpp, and check health
-    LibUPnP *mylib = 0;
-    string hwaddr;
-    int libretrysecs = 10;
-    for (;;) {
-        // Libupnp init fails if we're started at boot and the network
-        // is not ready yet. So retry this forever
-        mylib = LibUPnP::getLibUPnP(true, &hwaddr, iface, upnpip, upport);
-        if (mylib) {
-            break;
-        }
-        sleep(libretrysecs);
-        libretrysecs = MIN(2*libretrysecs, 120);
-    }
-
-    if (!mylib->ok()) {
-        LOGFAT("Lib init failed: " <<
-               mylib->errAsString("main", mylib->getInitError()) << endl);
-        return 1;
-    }
-
+    
+    // Initialise lower upnp lib logging. Static so can be done before
+    // the rest of init.
     if ((cp = getenv("UPMPDCLI_UPNPLOGFILENAME"))) {
         char *cp1 = getenv("UPMPDCLI_UPNPLOGLEVEL");
         int loglevel = LibUPnP::LogLevelNone;
@@ -511,26 +665,54 @@ int main(int argc, char *argv[])
             loglevel = atoi(cp1);
         }
         loglevel = loglevel < 0 ? 0: loglevel;
-        loglevel = loglevel > int(LibUPnP::LogLevelDebug) ? 
-            int(LibUPnP::LogLevelDebug) : loglevel;
-
         if (loglevel != LibUPnP::LogLevelNone) {
-            mylib->setLogFileName(cp, LibUPnP::LogLevel(loglevel));
+            LibUPnP::setLogFileName(cp, LibUPnP::LogLevel(loglevel));
         }
     }
 
-    // Create unique ID
-    string UUID = LibUPnP::makeDevUUID(friendlyname, hwaddr);
+    // Initialize libupnpp, and check health
+    LibUPnP *mylib = 0;
+    string hwaddr;
+    int libretrysecs = 10;
+    int flags = LibUPnP::UPNPPINIT_FLAG_SERVERONLY;;
+    if (!g_config->getBool("useipv6", false)) {
+        flags |= LibUPnP::UPNPPINIT_FLAG_NOIPV6;
+    }
+    for (;;) {
+        // Libupnp init fails if we're started at boot and the network
+        // is not ready yet. So retry this forever
+        if (LibUPnP::init(flags,
+                          LibUPnP::UPNPPINIT_OPTION_IFNAMES, &iface,
+                          LibUPnP::UPNPPINIT_OPTION_IPV4, &upnpip,
+                          LibUPnP::UPNPPINIT_OPTION_PORT, upport,
+                          LibUPnP::UPNPPINIT_OPTION_END)) {
+            break;
+        }
+        sleep(libretrysecs);
+        libretrysecs = MIN(2*libretrysecs, 120);
+    }
+    mylib = LibUPnP::getLibUPnP();
+    if (!mylib || !mylib->ok()) {
+        LOGFAT("Lib init failed: " <<
+               mylib->errAsString("main", mylib->getInitError()) << endl);
+        return 1;
+    }
+    hwaddr = mylib->hwaddr();
+    
+    // Create unique IDs for renderer and possible media server
+    if (!g_config || !g_config->get("msfriendlyname", fnameMS)) {
+        fnameMS = friendlyname + "-mediaserver";
+    }
+    uuidMS = LibUPnP::makeDevUUID(fnameMS, hwaddr);
 
-    // Initialize the data we serve through HTTP (device and service
-    // descriptions, icons, presentation page, etc.)
-    unordered_map<string, VDirContent> files;
-    if (!initHttpFs(files, g_datadir, UUID, friendlyname, enableAV, enableOH,
-                    !senderpath.empty(),
-                    iconpath, presentationhtml)) {
-        exit(1);
+    // If running as mediaserver only, make sure we don't conflict
+    // with a possible renderer
+    if (msonly) {
+        pidfilename = pidfilename + "-ms";
     }
 
+    opts.iconpath = iconpath;
+    opts.presentationhtml = presentationhtml;
     if (ownqueue)
         opts.options |= UpMpd::upmpdOwnQueue;
     if (enableOH)
@@ -541,6 +723,22 @@ int main(int argc, char *argv[])
         opts.sc2mpdpath = sc2mpdpath;
         opts.options |= UpMpd::upmpdOhReceiver;
     }
+    if (!screceiverstatefile.empty()) {
+        opts.screceiverstatefile = screceiverstatefile;
+        int fd;
+        if ((fd = open(opts.screceiverstatefile.c_str(),
+                       O_CREAT|O_RDWR, 0644)) < 0) {
+            LOGERR("creat(" << opts.screceiverstatefile << ") : errno : "
+                   << errno << endl);
+        } else {
+            close(fd);
+            if (geteuid() == 0 && chown(opts.screceiverstatefile.c_str(),
+                                        runas, -1) != 0) {
+                LOGERR("chown(" << opts.screceiverstatefile << ") : errno : "
+                       << errno << endl);
+            }
+        }
+    }
     if (!senderpath.empty()) {
         opts.options |= UpMpd::upmpdOhSenderReceiver;
         opts.senderpath = senderpath;
@@ -549,16 +747,53 @@ int main(int argc, char *argv[])
 
     if (!enableAV)
         opts.options |= UpMpd::upmpdNoAV;
-    // Initialize the UPnP device object.
-    UpMpd device(string("uuid:") + UUID, friendlyname, ohProductDesc,
-                 files, mpdclip, opts);
-    dev = &device;
 
-    // And forever generate state change events.
-    LOGDEB("Entering event loop" << endl);
     setupsigs();
-    device.eventloop();
-    LOGDEB("Event loop returned" << endl);
 
+    UpMpd *mediarenderer{nullptr};
+    if (!msonly) {
+        mediarenderer = new UpMpd(hwaddr, friendlyname,
+                                  ohProductDesc, mpdclip, opts);
+        UpMpdOpenHome *oh = mediarenderer->getoh();
+        // rootdevice is only used if we implement the media server as
+        // an embedded device, which is mostly for testing purposes
+        // and not done by default
+        if (nullptr != oh) {
+            rootdevice = oh;
+            devs.push_back(oh);
+        }
+        UpMpdMediaRenderer *av = mediarenderer->getav();
+        if (nullptr != av) {
+            rootdevice = av;
+            devs.push_back(av);
+        }
+    }
+
+    if (inprocessms && !startMediaServer(enableMediaServer)) {
+        LOGERR("Could not start media server\n");
+        std::cerr << "Could not start media server\n";
+        return 0;
+    }
+
+    if (!msonly) {
+        LOGDEB("Renderer event loop" << endl);
+        mediarenderer->startnoloops();
+        mpdclip->startEventLoop();
+    }
+
+    pause();
+    LOGDEB("Event loop returned" << endl);
     return 0;
 }
+
+// Read file from datadir
+bool readLibFile(const std::string& name, std::string& contents)
+{
+    string path = path_cat(g_datadir, name);
+    string reason;
+    if (!file_to_string(path, contents, &reason)) {
+        LOGERR("readLibFile: error reading " << name << " : " << reason << endl);
+        return false;
+    }
+    return true;
+}
