@@ -1,23 +1,23 @@
-/* Copyright (C) 2014 J.F.Dockes
- *	 This program is free software; you can redistribute it and/or modify
- *	 it under the terms of the GNU General Public License as published by
- *	 the Free Software Foundation; either version 2 of the License, or
- *	 (at your option) any later version.
+/* Copyright (C) 2014-2020 J.F.Dockes
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation; either version 2.1 of the License, or
+ *  (at your option) any later version.
  *
- *	 This program is distributed in the hope that it will be useful,
- *	 but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	 GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
- *	 You should have received a copy of the GNU General Public License
- *	 along with this program; if not, write to the
- *	 Free Software Foundation, Inc.,
- *	 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program; if not, write to the
+ *  Free Software Foundation, Inc.,
+ *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
-#include "ohinfo.hxx"
+#include "config.h"
 
-#include <upnp/upnp.h>
+#include "ohinfo.hxx"
 
 #include <functional>
 #include <iostream>
@@ -40,33 +40,42 @@ using namespace std::placeholders;
 static const string sTpProduct("urn:av-openhome-org:service:Info:1");
 static const string sIdProduct("urn:av-openhome-org:serviceId:Info");
 
-OHInfo::OHInfo(UpMpd *dev)
-    : OHService(sTpProduct, sIdProduct, dev), m_ohpl(0)
+OHInfo::OHInfo(UpMpd *dev, UpMpdOpenHome *udev, bool updstatus)
+    : OHService(sTpProduct, sIdProduct, "OHInfo.xml", dev, udev),
+      m_updstatus(updstatus)
 {
-    dev->addActionMapping(this, "Counters", 
-                          bind(&OHInfo::counters, this, _1, _2));
-    dev->addActionMapping(this, "Track", 
-                          bind(&OHInfo::track, this, _1, _2));
-    dev->addActionMapping(this, "Details", 
-                          bind(&OHInfo::details, this, _1, _2));
-    dev->addActionMapping(this, "Metatext", 
-                          bind(&OHInfo::metatext, this, _1, _2));
+    udev->addActionMapping(this, "Counters", 
+                           bind(&OHInfo::counters, this, _1, _2));
+    udev->addActionMapping(this, "Track", 
+                           bind(&OHInfo::track, this, _1, _2));
+    udev->addActionMapping(this, "Details", 
+                           bind(&OHInfo::details, this, _1, _2));
+    udev->addActionMapping(this, "Metatext", 
+                           bind(&OHInfo::metatext, this, _1, _2));
 }
 
 void OHInfo::urimetadata(string& uri, string& metadata)
 {
-    const MpdStatus &mpds =  m_dev->getMpdStatusNoUpdate();
+    const MpdStatus &mpds =  m_updstatus ?
+        m_dev->getMpdStatus() : m_dev->getMpdStatus();
     bool is_song = (mpds.state == MpdStatus::MPDS_PLAY) || 
         (mpds.state == MpdStatus::MPDS_PAUSE);
 
     if (is_song) {
-        uri = mpds.currentsong.uri;
-        // Prefer metadata from cache (copy from media server) to
-        // whatever comes from mpd
-        if (m_ohpl && m_ohpl->cacheFind(uri, metadata)) {
-            return;
+        uri = mpds.currentsong.rsrc.uri;
+        // If somebody (e.g. ohradio) took care to set the metatext, use it.
+        // Metatext is reset by OHProduct::setSourceIndex.
+        if (!m_metatext.empty()) {
+            metadata = m_metatext;
+        } else {
+            // Playlist or AVTransport playing, probably.
+            // Prefer metadata from cache (copy from media server) to
+            // whatever comes from mpd.
+            if (m_ohpl && m_ohpl->cacheFind(uri, metadata)) {
+                return;
+            }
+            metadata = didlmake(mpds.currentsong);
         }
-        metadata = didlmake(mpds.currentsong);
     } else {
         uri.clear();
         metadata.clear();
@@ -76,7 +85,7 @@ void OHInfo::urimetadata(string& uri, string& metadata)
 void OHInfo::makedetails(string &duration, string& bitrate, 
                          string& bitdepth, string& samplerate)
 {
-    const MpdStatus &mpds =  m_dev->getMpdStatusNoUpdate();
+    const MpdStatus &mpds =  m_dev->getMpdStatus();
 
     bool is_song = (mpds.state == MpdStatus::MPDS_PLAY) || 
         (mpds.state == MpdStatus::MPDS_PAUSE);
@@ -95,11 +104,10 @@ bool OHInfo::makestate(unordered_map<string, string> &st)
 {
     st.clear();
 
-    st["TrackCount"] = SoapHelp::i2s(m_dev->m_mpds ? 
-                                     m_dev->m_mpds->trackcounter : 0);
-    st["DetailsCount"] = SoapHelp::i2s(m_dev->m_mpds ? 
-                                       m_dev->m_mpds->detailscounter : 0);
-    st["MetatextCount"] = "0";
+    st["TrackCount"] = SoapHelp::i2s(m_dev->getMpdStatus().trackcounter);
+    st["DetailsCount"] =
+        SoapHelp::i2s(m_dev->getMpdStatus().detailscounter);
+    st["MetatextCount"] = SoapHelp::i2s(m_metatextcnt);
     string uri, metadata;
     urimetadata(uri, metadata);
     st["Uri"] = uri;
@@ -115,11 +123,11 @@ int OHInfo::counters(const SoapIncoming& sc, SoapOutgoing& data)
 {
     LOGDEB("OHInfo::counters" << endl);
     
-    data.addarg("TrackCount", SoapHelp::i2s(m_dev->m_mpds ?
-                                            m_dev->m_mpds->trackcounter : 0));
-    data.addarg("DetailsCount", SoapHelp::i2s(m_dev->m_mpds ?
-                                              m_dev->m_mpds->detailscounter:0));
-    data.addarg("MetatextCount", "0");
+    data.addarg("TrackCount",
+                SoapHelp::i2s(m_dev->getMpdStatus().trackcounter));
+    data.addarg("DetailsCount", SoapHelp::i2s(
+                    m_dev->getMpdStatus().detailscounter));
+    data.addarg("MetatextCount", SoapHelp::i2s(m_metatextcnt));
     return UPNP_E_SUCCESS;
 }
 
@@ -159,5 +167,9 @@ int OHInfo::metatext(const SoapIncoming& sc, SoapOutgoing& data)
 void OHInfo::setMetatext(const string& metatext)
 {
     //LOGDEB1("OHInfo::setMetatext: " << metatext << endl);
-    m_metatext = metatext;
+    if (metatext.compare(m_metatext)) {
+        m_metatext = metatext;
+        m_metatextcnt++;
+        onEvent(nullptr);
+    }
 }
