@@ -1,39 +1,40 @@
-/* Copyright (C) 2014 J.F.Dockes
- *	 This program is free software; you can redistribute it and/or modify
- *	 it under the terms of the GNU General Public License as published by
- *	 the Free Software Foundation; either version 2 of the License, or
- *	 (at your option) any later version.
+/* Copyright (C) 2014-2020 J.F.Dockes
+ *  This program is free software; you can redistribute it and/or modify
+ *  it under the terms of the GNU Lesser General Public License as published by
+ *  the Free Software Foundation; either version 2.1 of the License, or
+ *  (at your option) any later version.
  *
- *	 This program is distributed in the hope that it will be useful,
- *	 but WITHOUT ANY WARRANTY; without even the implied warranty of
- *	 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- *	 GNU General Public License for more details.
+ *  This program is distributed in the hope that it will be useful,
+ *  but WITHOUT ANY WARRANTY; without even the implied warranty of
+ *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ *  GNU Lesser General Public License for more details.
  *
- *	 You should have received a copy of the GNU General Public License
- *	 along with this program; if not, write to the
- *	 Free Software Foundation, Inc.,
- *	 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ *  You should have received a copy of the GNU Lesser General Public License
+ *  along with this program; if not, write to the
+ *  Free Software Foundation, Inc.,
+ *  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
 
+#include "config.h"
+
 #include "renderctl.hxx"
 
-#include <stdlib.h>                     // for atoi
-#include <upnp/upnp.h>                  // for UPNP_E_INVALID_PARAM, etc
+#include <stdlib.h>
 
-#include <functional>                   // for _Bind, bind, _1, _2
-#include <iostream>                     // for basic_ostream::operator<<, etc
-#include <map>                          // for _Rb_tree_const_iterator, etc
-#include <string>                       // for string, basic_string
-#include <unordered_map>                // for unordered_map, etc
-#include <utility>                      // for pair
-#include <vector>                       // for vector
+#include <functional>
+#include <iostream>
+#include <map>
+#include <string>
+#include <unordered_map>
+#include <utility>
+#include <vector>
 
-#include "libupnpp/log.hxx"             // for LOGDEB
-#include "libupnpp/soaphelp.hxx"        // for SoapIncoming, SoapOutgoing, i2s, etc
+#include "libupnpp/log.hxx"
+#include "libupnpp/soaphelp.hxx"
 
-#include "mpdcli.hxx"                   // for MPDCli, MpdStatus
-#include "upmpd.hxx"                    // for UpMpd
-#include "upmpdutils.hxx"               // for dbvaluetopercent, mapget, etc
+#include "mpdcli.hxx"
+#include "upmpd.hxx"
+#include "upmpdutils.hxx"
 
 using namespace std;
 using namespace std::placeholders;
@@ -43,24 +44,43 @@ static const string
 sTpRender("urn:schemas-upnp-org:service:RenderingControl:1");
 static const string sIdRender("urn:upnp-org:serviceId:RenderingControl");
 
-static const int minVolumeDelta = 5;
-
-UpMpdRenderCtl::UpMpdRenderCtl(UpMpd *dev, bool noev)
-    : UpnpService(sTpRender, sIdRender, dev, noev), m_dev(dev), 
-      m_desiredvolume(-1)
+UpMpdRenderCtl::UpMpdRenderCtl(UpMpd *dev, UpMpdMediaRenderer* udev, bool noev)
+    : UpnpService(sTpRender, sIdRender, "RenderingControl.xml", udev, noev),
+      m_dev(dev), m_udev(udev)
 {
-    m_dev->addActionMapping(this, "SetMute", 
+    m_udev->addActionMapping(this, "SetMute", 
                             bind(&UpMpdRenderCtl::setMute, this, _1, _2));
-    m_dev->addActionMapping(this, "GetMute", 
+    m_udev->addActionMapping(this, "GetMute", 
                             bind(&UpMpdRenderCtl::getMute, this, _1, _2));
-    m_dev->addActionMapping(this, "SetVolume", bind(&UpMpdRenderCtl::setVolume, 
+    m_udev->addActionMapping(this, "SetVolume", bind(&UpMpdRenderCtl::setVolume, 
                                               this, _1, _2, false));
-    m_dev->addActionMapping(this, "GetVolume", bind(&UpMpdRenderCtl::getVolume, 
+    m_udev->addActionMapping(this, "GetVolume", bind(&UpMpdRenderCtl::getVolume, 
                                               this, _1, _2, false));
-    m_dev->addActionMapping(this, "ListPresets", 
+    m_udev->addActionMapping(this, "ListPresets", 
                             bind(&UpMpdRenderCtl::listPresets, this, _1, _2));
-    m_dev->addActionMapping(this, "SelectPreset", 
+    m_udev->addActionMapping(this, "SelectPreset", 
                             bind(&UpMpdRenderCtl::selectPreset, this, _1, _2));
+
+    m_dev->getmpdcli()->subscribe(MPDCli::MpdMixerEvt,
+                               bind(&UpMpdRenderCtl::onMpdEvent, this, _1));
+}
+
+// Rendering Control errors
+enum RDCErrorCode {
+  UPNP_AV_RC_INVALID_PRESET_NAME                    = 701,
+  UPNP_AV_RC_INVALID_INSTANCE_ID                    = 702,
+};
+
+const std::string UpMpdRenderCtl::serviceErrString(int error) const
+{
+    switch(error) {
+    case UPNP_AV_RC_INVALID_PRESET_NAME:
+        return "Rendering Control Invalid Preset Name";
+    case UPNP_AV_RC_INVALID_INSTANCE_ID:
+        return "Rendering Control Invalid Instance ID";
+    default:
+        return "Rendering Control Unknown Error";
+    }
 }
 
 ////////////////////////////////////////////////////
@@ -83,9 +103,7 @@ UpMpdRenderCtl::UpMpdRenderCtl(UpMpd *dev, bool noev)
 
 bool UpMpdRenderCtl::rdstateMToU(unordered_map<string, string>& status)
 {
-    const MpdStatus &mpds = m_dev->getMpdStatusNoUpdate();
-
-    int volume = m_desiredvolume >= 0 ? m_desiredvolume : mpds.volume;
+    int volume = m_dev->getvolume();
     if (volume < 0)
         volume = 0;
     status["Volume"] = SoapHelp::i2s(volume);
@@ -97,13 +115,13 @@ bool UpMpdRenderCtl::rdstateMToU(unordered_map<string, string>& status)
 bool UpMpdRenderCtl::getEventData(bool all, std::vector<std::string>& names, 
                                   std::vector<std::string>& values)
 {
-    //LOGDEB("UpMpdRenderCtl::getEventDataRendering. desiredvolume " << 
-    //		   m_desiredvolume << (all?" all " : "") << endl);
-    if (m_desiredvolume >= 0) {
-        m_dev->m_mpdcli->setVolume(m_desiredvolume);
-        m_desiredvolume = -1;
-    }
+    m_dev->flushvolume();
+    return getEventDataNoFlush(all, names, values);
+}
 
+bool UpMpdRenderCtl::getEventDataNoFlush(
+    bool all, std::vector<std::string>& names, std::vector<std::string>& values)
+{
     unordered_map<string, string> newstate;
     rdstateMToU(newstate);
     if (all)
@@ -125,6 +143,9 @@ bool UpMpdRenderCtl::getEventData(bool all, std::vector<std::string>& names,
 
         chgdata += "<";
         chgdata += it->first;
+        if (!it->first.compare("Volume") || !it->first.compare("Mute")) {
+            chgdata += " channel=\"Master\"";
+        }
         chgdata += " val=\"";
         chgdata += SoapHelp::xmlQuote(it->second);
         chgdata += "\"/>\n";
@@ -143,6 +164,16 @@ bool UpMpdRenderCtl::getEventData(bool all, std::vector<std::string>& names,
     return true;
 }
 
+void UpMpdRenderCtl::onMpdEvent(const MpdStatus*)
+{
+    LOGDEB0("RenderCtl::onMpdEvent()\n");
+    std::vector<std::string> names, values;
+    getEventDataNoFlush(false, names, values);
+    if (!names.empty()) {
+        m_udev->notifyEvent(this, names, values);
+    }
+}
+
 // Actions:
 // Note: we need to return all out arguments defined by the SOAP call even if
 // they don't make sense (because there is no song playing). Ref upnp arch p.51:
@@ -173,41 +204,6 @@ int UpMpdRenderCtl::getVolumeDBRange(const SoapIncoming& sc, SoapOutgoing& data)
 }
 #endif
 
-int UpMpdRenderCtl::getvolume_i()
-{
-    return m_desiredvolume >= 0 ? m_desiredvolume : 
-        m_dev->m_mpdcli->getVolume();
-}
-
-void UpMpdRenderCtl::setvolume_i(int volume)
-{
-    int previous_volume = m_dev->m_mpdcli->getVolume();
-    int delta = previous_volume - volume;
-    if (delta < 0)
-        delta = -delta;
-    LOGDEB("UpMpdRenderCtl::setVolume: volume " << volume << " delta " << 
-           delta << endl);
-    if (delta >= minVolumeDelta) {
-        m_dev->m_mpdcli->setVolume(volume);
-        m_desiredvolume = -1;
-    } else {
-        m_desiredvolume = volume;
-    }
-}
-
-void UpMpdRenderCtl::setmute_i(bool onoff)
-{
-    if (onoff) {
-        if (m_desiredvolume >= 0) {
-            m_dev->m_mpdcli->setVolume(m_desiredvolume);
-            m_desiredvolume = -1;
-        }
-        m_dev->m_mpdcli->setVolume(0, true);
-    } else {
-        // Restore pre-mute
-        m_dev->m_mpdcli->setVolume(1, true);
-    }
-}
 
 int UpMpdRenderCtl::setMute(const SoapIncoming& sc, SoapOutgoing& data)
 {
@@ -220,13 +216,12 @@ int UpMpdRenderCtl::setMute(const SoapIncoming& sc, SoapOutgoing& data)
         return UPNP_E_INVALID_PARAM;
     }
     if (desired[0] == 'F' || desired[0] == '0') {
-        setmute_i(false);
+        m_dev->setmute(false);
     } else if (desired[0] == 'T' || desired[0] == '1') {
-        setmute_i(true);
+        m_dev->setmute(true);
     } else {
         return UPNP_E_INVALID_PARAM;
     }
-    m_dev->loopWakeup();
     return UPNP_E_SUCCESS;
 }
 
@@ -237,12 +232,13 @@ int UpMpdRenderCtl::getMute(const SoapIncoming& sc, SoapOutgoing& data)
         return UPNP_E_INVALID_PARAM;
     }
 
-    int volume = m_dev->m_mpdcli->getVolume();
+    int volume = m_dev->getvolume();
     data.addarg("CurrentMute", volume == 0 ? "1" : "0");
     return UPNP_E_SUCCESS;
 }
 
-int UpMpdRenderCtl::setVolume(const SoapIncoming& sc, SoapOutgoing& data, bool isDb)
+int UpMpdRenderCtl::setVolume(const SoapIncoming& sc, SoapOutgoing& data,
+                              bool isDb)
 {
     string channel;
     if (!sc.get("Channel", &channel) || channel.compare("Master")) {
@@ -261,22 +257,21 @@ int UpMpdRenderCtl::setVolume(const SoapIncoming& sc, SoapOutgoing& data, bool i
     if (volume < 0 || volume > 100) {
         return UPNP_E_INVALID_PARAM;
     }
-	
-    setvolume_i(volume);
-
-    m_dev->loopWakeup();
+    
+    m_dev->setvolume(volume);
     return UPNP_E_SUCCESS;
 }
 
-int UpMpdRenderCtl::getVolume(const SoapIncoming& sc, SoapOutgoing& data, bool isDb)
+int UpMpdRenderCtl::getVolume(const SoapIncoming& sc, SoapOutgoing& data,
+                              bool isDb)
 {
     // LOGDEB("UpMpdRenderCtl::getVolume" << endl);
     string channel;
     if (!sc.get("Channel", &channel) || channel.compare("Master")) {
         return UPNP_E_INVALID_PARAM;
     }
-	
-    int volume = getvolume_i();
+    
+    int volume = m_dev->getvolume();
     if (isDb) {
         volume = percentodbvalue(volume);
     }
@@ -304,7 +299,7 @@ int UpMpdRenderCtl::selectPreset(const SoapIncoming& sc, SoapOutgoing& data)
 
     // Well there is only the volume actually...
     int volume = 50;
-    m_dev->m_mpdcli->setVolume(volume);
+    m_dev->setvolume(volume);
 
     return UPNP_E_SUCCESS;
 }
